"""
Verifying Safety Properties
"""

from typing import Tuple, Optional
from mc308.auto.ts import Action, TS, FinitePath, State, Trans
from mc308.auto.nfa import NFA
from mc308.logics.bool import BoolFormula

from pysmt.shortcuts import Symbol, And, Not, Or

from typing import Collection, Set, List
import itertools


def nfa_intersection(nfa1 : NFA, nfa2 : NFA) -> NFA:
    """
    Implement the intersection (synchronous product) of two NFAs

    This function returns a NFA A such that:
    Language(A) = Language(nfa1) and Language(nfa2)
    """

    raise NotImplementedError("")



def ts_nfa_product(ts : TS, nfa : NFA) -> TS:
    """
    Implement the intersection of a TS with a NFA nfa.

    The result is a new transition system TS1 such that:
    Traces_fin(TS1) = Traces_fin(TS) and Language(nfa)
    """
    raise NotImplementedError("TS and NFA intersection not implemented")



def check_safety(ts : TS, bad_prefixes : NFA) -> Tuple[bool, Optional[FinitePath]]:
    """
    bad_prefixes describes is a (regular) set of bad prefixes for a safety
    property P_safe

    This function checks if ts |= P_safe and returns either:
    - (True, None) if ts |= safety
    - (False, cex)    otherwise
      cex is a sequence of states of ts starting from the initial state and
      reaching a state s that does not satisfy the safety property.
    """
    raise NotImplementedError("check_safety not implemented")

